//
//  ========================================================================
//  Copyright (c) 1995-2016 Mort Bay Consulting Pty. Ltd.
//  ------------------------------------------------------------------------
//  All rights reserved. This program and the accompanying materials
//  are made available under the terms of the Eclipse Public License v1.0
//  and Apache License v2.0 which accompanies this distribution.
//
//      The Eclipse Public License is available at
//      http://www.eclipse.org/legal/epl-v10.html
//
//      The Apache License v2.0 is available at
//      http://www.opensource.org/licenses/apache2.0.php
//
//  You may elect to redistribute this code under either of these licenses.
//  ========================================================================
//

package org.eclipse.jetty.util.log;

import android.util.Log;

import org.beigesoft.log.LogFileAdp;
import org.beigesoft.accandr.SrvAccJet;

/**
 * Logger into file only for all classes.
 * File with name "ajetty[0/1].log" is placed in directory
 * where A-Jetty jar file is invoked.
 * This is not suitable for Android.
 * Many software uses Slf4J, e.g. HikariCP, so this is not suitable
 * for that situation. But A-Jetty for Android not uses HikariCP,
 * so it's best way for it.
 */
public class LoggerFileAndroid extends AILoggerLog {

  /**
   * <p>Create logger for all classes.</p>
   **/
  @Override
  protected final void createLogger() {
    LogFileAdp logger = new LogFileAdp();
    String dir = SrvAccJet.CNTX.getFilesDir().getAbsolutePath();
    logger.setPath(dir + "/ajetty-eis");
    Log.i("A-Jetty", "> Log file path: " + logger.getPath());
    setLogger(logger);
  }
}
